;****************************************Copyright (c)****************************************************
;**                               Guangzou ZLG-MCU Development Co.,LTD.
;**                                      graduate school
;**                                 http://www.zlgmcu.com
;**
;**--------------File Info--------------------------------------------------------------------------------
;** File name: 	            os_cpu_a.asm
;** Last modified Date:     2007.12.12
;** Last Version:           1.0
;** Descriptions: 	    The assembly functions that a uC/OS-II port requires
;**                         uC/OS-IIֲҪĻຯ  
;**-------------------------------------------------------------------------------------------------------
;** Created By:       	    Steven Zhou ܸ
;** Created date:   	    2007.12.12
;** Version:	            1.0
;** Descriptions:           The original version
;**
;**-------------------------------------------------------------------------------------------------------
;** Modified by:            
;** Modified date:          
;** Version:                
;** Description:            
;**
;*********************************************************************************************************

;*********************************************************************************************************
;  Declarations for the interrupt handlers that are used by the application.              
;  Ӧõ                                       
;*********************************************************************************************************
    EXTERN  OSRunning                
    EXTERN  OSPrioCur
    EXTERN  OSPrioHighRdy
    EXTERN  OSTCBCur
    EXTERN  OSTCBHighRdy
    EXTERN  OSIntNesting
    EXTERN  OSIntExit
    EXTERN  OSTaskSwHook
    EXTERN  OsEnterSum


;*********************************************************************************************************
;  Declarations for the exported functions             
;  ⲿ                          
;*********************************************************************************************************
    EXPORT  OSStartHighRdy
    EXPORT  OSCtxSw
    EXPORT  OSIntCtxSw
    EXPORT  OSPendSV
    EXPORT  OS_ENTER_CRITICAL
    EXPORT  OS_EXIT_CRITICAL                  
    EXPORT  intDisAll
           
           
;*********************************************************************************************************
;  Registers or macros used by this file                                
;  ļõļĴͺ                             
;*********************************************************************************************************							    
NVIC_INT_CTRL           EQU     0xE000ED04                              ;  Interrupt control state  
                                                                        ;  register.
                                                                        ;  жϿƼĴ

NVIC_SYSPRI2            EQU     0xE000ED20                              ;  System priority register (2)
                                                                        ;  ϵͳȼĴ2

NVIC_PENDSV_PRI         EQU     0xFFFF0000                              ;  PendSV and system tick priority
                                                                        ;  (Both are lowest,0xff)  
                                                                        ;  жϺϵͳж
                                                                        ;  Ϊͣ0xff.
NVIC_PENDSVSET          EQU     0x10000000                              ;  Value to trigger PendSV  
                                                                        ;  exception.жϵֵ.

OS_CRITICAL_INT_PRIO    EQU     (1 << 5)		                ;  Critical setions manage priority 
                                                                        ;  (equal or bigger number managed) 
                                                                        ;  ٽȼֵ
                                                                        ;  ڱ
	         
		
    RSEG CODE:CODE:NOROOT(2)
    
;*********************************************************************************************************
;** Function name:      OS_ENTER_CRITICAL
;** Descriptions:       Enter the critical section  ٽ                         
;** Input parameters:	None 
;** Output parameters:	None 
;** Returned value:	None 
;** Created by:		Steven Zhou ܸ
;** Created Date:	2007.12.12
;**-------------------------------------------------------------------------------------------------------
;** Modified by:
;** Modified date:
;**-------------------------------------------------------------------------------------------------------
;*********************************************************************************************************
OS_ENTER_CRITICAL
    IF OS_CRITICAL_INT_PRIO > 0
     	PUSH 	{R1,R2} 	                
     	LDR 	R1, =OS_CRITICAL_INT_PRIO                               ;  Disable priorities between 
                                                                        ;  OS_CRITICAL_INT_PRIO & 0xFF 
                                                                             			 				
        MSR 	BASEPRI,  R1                                            ;  ȼOS_CRITICAL_INT 
                                                                        ;  _PRIO0xFFж      			 				 
    ELSE
	CPSID   I                                                       ;  Disable all the interrupts
                                                                        ;  ж
        PUSH 	{R1,R2}      
    ENDIF
    
    LDR     R1, __OS_EnterSum	                                        ;  OsEnterSum++
    LDRB    R2, [R1]
    ADD     R2, R2, #1
    STRB    R2, [R1]
    POP     {R1,R2}
    BX      LR


;*********************************************************************************************************
;** Function name:      OS_EXIT_CRITICAL
;** Descriptions:       Exit the critical section  ˳ٽ
;** Output parameters:	None 
;** Input parameters:	None 
;** Returned value:	None 
;** Created by:		Steven Zhou ܸ
;** Created Date:       2007.12.12
;**-------------------------------------------------------------------------------------------------------
;** Modified by:    
;** Modified date:  
;**-------------------------------------------------------------------------------------------------------
;*********************************************************************************************************
OS_EXIT_CRITICAL
	PUSH    {R1, R2}
	LDR     R1, __OS_EnterSum                                     	;  OsEnterSum--
        LDRB    R2, [R1]
        SUB     R2, R2, #1
        STRB    R2, [R1]
	MOV     R1,  #0    
	CMP     R2,  #0			                                ;  if OsEnterSum=0,enable 
                                                                        ;  interrupts.  
     	                                                                ;  OsEnterSum=0жϡ
        
        IF OS_CRITICAL_INT_PRIO > 0 
 	    IT EQ 
  	    MSREQ  BASEPRI, R1 
	ELSE
            IT EQ 
            MSREQ  PRIMASK, R1     
	ENDIF
        
        POP     {R1, R2}
        BX      LR
        

;*********************************************************************************************************
;** Function name:      OSStartHighRdy
;** Descriptions:	Uses the context switcher  to cause the first task to start.
;**                     ʹõеһ   
;** Input parameters:	None 
;** Output parameters:	None 
;** Returned value:	None 
;** Created by:		Steven Zhou ܸ
;** Created Date:	2007.12.12
;**-------------------------------------------------------------------------------------------------------
;** Modified by:            
;** Modified date:          
;**-------------------------------------------------------------------------------------------------------
;*********************************************************************************************************
OSStartHighRdy

    LDR     R4, =NVIC_SYSPRI2                                           ;  set the PendSV exception 
                                                                        ;  priorityPendSVȼ
    LDR     R5, =NVIC_PENDSV_PRI	
    STR     R5, [R4]
	    
    MOV     R4, #0                                                      ;  set the PSP to 0 for initial  
                                                                        ;  context switch call ʹPSP0
    MSR     PSP, R4

    LDR     R4, __OS_Running                                            ;  OSRunning = TRUE
    MOV     R5, #1
    STRB    R5, [R4]

    LDR     R4, =NVIC_INT_CTRL                                          ;  trigger the PendSV exception
                                                                        ;  ж
    LDR     R5, =NVIC_PENDSVSET
    STR     R5, [R4]

    CPSIE   I                                                           ;  enable interrupts at processor 
                                                                        ;  levelʹȼж
OSStartHang
    B       OSStartHang             


;*********************************************************************************************************
;** Function name:      OSCtxSw
;** Descriptions:	Perform a contex switch from task level  л                          
;** Input parameters:	None 
;** Output parameters:  None 
;** Returned value:	None         
;** Created by:		Steven Zhou ܸ
;** Created Date:       2007.12.12
;**-------------------------------------------------------------------------------------------------------
;** Modified by:           
;** Modified date:         
;**-------------------------------------------------------------------------------------------------------
;*********************************************************************************************************
OSCtxSw
    PUSH    {R4, R5}
    LDR     R4, =NVIC_INT_CTRL                                          ;  trigger the PendSV exception
                                                                        ;  ж
    LDR     R5, =NVIC_PENDSVSET
    STR     R5, [R4]
    POP     {R4, R5}
    BX      LR


;*********************************************************************************************************
;** Function name:      OSIntCtxSw
;** Descriptions:	Called by OSIntExit() when it determines a context switch is needed as the
;**                     result of an interrupt.
;**                     жϼл                 
;** Input parameters:   None 
;** Output parameters:	None 
;** Returned value:	None 
;** Created by:		Steven Zhou ܸ
;** Created Date:       2007.12.12
;**-------------------------------------------------------------------------------------------------------
;** Modified by:   
;** Modified date: 
;**-------------------------------------------------------------------------------------------------------
;*********************************************************************************************************
OSIntCtxSw
    PUSH    {R4, R5}
    LDR     R4, =NVIC_INT_CTRL                                          ;  trigger the PendSV exception
                                                                        ;  ж
    LDR     R5, =NVIC_PENDSVSET
    STR     R5, [R4]
    POP     {R4, R5}
    BX      LR
    NOP


;*********************************************************************************************************
;** Function name:      OSPendSV
;** Descriptions:	Used to cause a context switch л
;** Input parameters:	None 
;** Output parameters:	None 
;** Returned value:	None 
;** Created by:		Steven Zhou ܸ
;** Created Date:	2007.12.12
;**-------------------------------------------------------------------------------------------------------
;** Modified by:         
;** Modified date:       
;**-------------------------------------------------------------------------------------------------------
;*********************************************************************************************************
OSPendSV
    IF OS_CRITICAL_INT_PRIO > 0	                                        ;  disable interupt ж
        MRS R3, BASEPRI
        LDR R1, =OS_CRITICAL_INT_PRIO
	MSR BASEPRI, R1
    ELSE
       	MRS     R3, PRIMASK                
       	CPSID   I
    ENDIF

    MRS     R0, PSP                                                     ;  PSP is process stack pointer  
                                                                        ;  PSPĶջָ
    CBZ     R0, OSPendSV_nosave                                         ;  skip register save the first 
                                                                        ;  timeһ

    SUB     R0, R0, #0x20                                               ;  save remaining regs r4-11 on 
                                                                        ;  process stack r4-r11
    STM     R0, {R4-R11}

    LDR     R4, __OS_TCBCur                                             ;  OSTCBCur->OSTCBStkPtr = SP;
    LDR     R4, [R4]
    STR     R0, [R4]                                                    ;  R0 is SP of process being 
                                                                        ;  switched outR0Ǳл
                                                                        ;  Ķջָ
OSPendSV_nosave
    PUSH    {R14}                                                       ;  need to save LR exc_return 
                                                                        ;  valueLRֵ

    LDR.W   R0, __OS_TaskSwHook                                         ;  OSTaskSwHook();
    BLX     R0
    POP     {R14}
 	  
    LDR     R4, __OS_PrioCur                                            ;  OSPrioCur = OSPrioHighRdy
    LDR     R5, __OS_PrioHighRdy     
    LDRB    R6, [R5]
    STRB    R6, [R4]

    LDR     R4, __OS_TCBCur                                             ;  OSTCBCur  = OSTCBHighRdy;
    LDR     R6, __OS_TCBHighRdy      
    LDR     R6, [R6]
    STR     R6, [R4]

    LDR     R0, [R6]                                                    ;  SP = OSTCBHighRdy->OSTCBStkPtr;
    LDM     R0, {R4-R11}                                                ;  restore r4-11 from new process
                                                                        ;  stack Ķջָr4-r11
    ADD     R0, R0, #0x20
    MSR     PSP, R0                                                     ;  load PSP with new process SP
                                                                        ;  ĶջָPSP
 	   	
    ORR     LR, LR, #0x04                                               ;  ensure exception return uses 
                                                                        ;  PSPȷغʹPSP
        
    IF OS_CRITICAL_INT_PRIO > 0                                         ;  restore interrupts ָж  
        MSR 	BASEPRI,  R3
    ELSE
	MSR     PRIMASK, R3
    ENDIF
		   
    BX      LR                                                          ; exception return will restore 
                                                                        ;  remaining context 
                                                                        ;  ʱָʣµ
    NOP


;*********************************************************************************************************
;** Function name:      IntDisAll
;** Descriptions:	Disable all interrupts from the interrupt controller  رжϿж
;** Input parameters:	None 
;** Output parameters:	None 
;** Returned value:	None 
;** Created by:		Steven Zhou ܸ
;** Created Date:	2007.12.12
;**-------------------------------------------------------------------------------------------------------
;** Modified by:         
;** Modified date:       
;**-------------------------------------------------------------------------------------------------------
;*********************************************************************************************************
intDisAll
    CPSID   I
    BX      LR
    
    
;*********************************************************************************************************
;  POINTERS TO VARIABLES
;  ָ  
;*********************************************************************************************************
    DATA

__OS_TaskSwHook:
    DC32    OSTaskSwHook

__OS_IntExit:
    DC32    OSIntExit

__OS_IntNesting:
    DC32    OSIntNesting

__OS_PrioCur:
    DC32    OSPrioCur

__OS_PrioHighRdy:
    DC32    OSPrioHighRdy

__OS_Running:
    DC32    OSRunning

__OS_TCBCur:
    DC32    OSTCBCur

__OS_TCBHighRdy:
    DC32    OSTCBHighRdy

__OS_EnterSum:
    DC32    OsEnterSum


    END

;*********************************************************************************************************
;  __END FILE 
;*********************************************************************************************************
